/* 

dvda-author  - Author a DVD-Audio DVD

(C) Dave Chapman <dave@dchapman.com> 2005

The latest version can be found at http://dvd-audio.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <sys/stat.h>
#include <sys/types.h>

#ifdef __WIN32__
  #include <io.h>
#endif

#include "version.h"
#include "audio.h"
#include "ats.h"
#include "atsi.h"
#include "amg.h"
#include "samg.h"

int main(int argc, char* argv[]) {
  int i,j;
  int error;
  int ngroups;
  int in_group;
  int ntracks[9];
  int atsi_sectors[9];
  int startsector=281;
  char outdir[512];
  char audiotsdir[540];
  char videotsdir[540];
  fileinfo_t files[9][99];
  uint64_t totalsize;

  fprintf(stderr,"dvda-author v" VERSION " - Copyright (C) 2005 Dave Chapman\n");
  fprintf(stderr,"Latest version available from http://dvd-audio.sourceforge.net/\n\n");
  fprintf(stderr,"This is free software; see the source for copying conditions.  There is NO\n");
  fprintf(stderr,"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");

  if (argc==1) {
    fprintf(stderr,"USAGE: dvda-author -pp NNN -o OUTPUT_DIRECTORY -g files ... [-g files ... ]\n");
    fprintf(stderr,"\nYou may specify up to 9 groups of tracks using the -g option.\n");
    fprintf(stderr,"-pp NNN specifies the number of the first sector of the AUDIO_PP.IFO file in the output of mkisofs\n");
    fprintf(stderr,"\nSupported audio types: .wav .flac\n\n");
    return(0);
  }

  outdir[0]=0;
  memset(ntracks,0,sizeof(ntracks));
  ngroups=0;
  in_group=0;
  for (i=1;i<argc;i++) {
    if (argv[i][0]=='-') {
      if (strcmp(argv[i],"-o")==0) {
        i++;
        if (i < argc) { 
          strncpy(outdir,argv[i],sizeof(outdir));
        }
      } else if (strcmp(argv[i],"-pp")==0) {
        i++;
        if (i < argc) { 
          startsector=atoi(argv[i]);
          fprintf(stderr,"INFO: Using starting sector of %d\n",startsector);
        }
      } else if (strcmp(argv[i],"-g")==0) {
        if ((in_group) && (ntracks[ngroups-1]==0)) {
          fprintf(stderr,"ERR: Can not create empty group (-g -g)\n");
          return(0);
        }
        in_group=1;
        if (ngroups < 9) {
          ngroups++;
        } else {
          fprintf(stderr,"ERR: DVD-Audio only supports up to 9 groups\n");
          return(0);
        }
      } else {
        fprintf(stderr,"ERR: Unknown option %s\n",argv[i]);
        return(0);
      }
    } else {
      if (in_group==0) {
        fprintf(stderr,"ERR: Error parsing command-line parameters\n");
        return(0);
      }

      if ((in_group) && (ngroups <= 9) && (ntracks[ngroups-1] < 99)) {
        files[ngroups-1][ntracks[ngroups-1]].filename=argv[i];
        ntracks[ngroups-1]++;
      } else {
        fprintf(stderr,"ERR: Too many input files specified - group %d, track %d\n",ngroups,ntracks[ngroups-1]);
        return(0);
      }
    }
  }

  if (outdir[0]==0) {
    fprintf(stderr,"ERR: No output directory specified\n"); 
    return(0);
  }

#ifdef __WIN32__
  error=mkdir(outdir);
#else
  error=mkdir(outdir,0777);
#endif
  if (error!=0) {
    fprintf(stderr,"WARNING: Could not create directory %s - does it already exist?\n",outdir);
  }

  snprintf(audiotsdir,sizeof(audiotsdir),"%s/AUDIO_TS",outdir);
  snprintf(videotsdir,sizeof(videotsdir),"%s/VIDEO_TS",outdir);
#ifdef __WIN32__
  error=mkdir(audiotsdir);
  error=mkdir(videotsdir);
#else
  error=mkdir(audiotsdir,0777);
  error=mkdir(videotsdir,0777);
#endif
  
  /* Step 1 - parse all audio files and store the file formats, lengths etc */
  fprintf(stderr,"DVD Layout:\n\n");
  fprintf(stderr,"Group  Track    Rate Bits  Ch       Length  Filename\n");
  for (i=0;i<ngroups;i++) {
    for (j=0;j<ntracks[i];j++) {
       error=wav_getinfo(&files[i][j]);
       if (error) {
          error=flac_getinfo(&files[i][j]);
       }
       if (error) {
          fprintf(stderr,"ERR: Can not open %s\n",files[i][j].filename);
          return(0);
       }
       fprintf(stderr,"    %d     %02d  %6d   %02d   %d   %10lld   ",i+1,j+1,files[i][j].samplerate,files[i][j].bitspersample,files[i][j].channels,files[i][j].numsamples);
       fprintf(stderr,"%s\n",files[i][j].filename);
       totalsize+=files[i][j].numbytes;
    }
  }
  fprintf(stderr,"\n");
  fprintf(stderr,"Size of raw PCM data: %lld bytes (%lldMB)\n\n",totalsize,totalsize/(1024*1024));

  for (i=0;i<ngroups;i++) {
    error=create_ats(audiotsdir,i+1,&files[i][0],ntracks[i]);
    error=create_atsi(audiotsdir,i+1,&files[i][0],ntracks[i],&atsi_sectors[i]);
  }
  error=create_amg(audiotsdir,&files[0][0],ntracks,ngroups,atsi_sectors);
  error=create_samg(audiotsdir,1,&files[0][0],ntracks,ngroups,atsi_sectors,startsector);

  fprintf(stderr,"\n");

  fprintf(stderr,"Group  Track    First Sect   Last Sect  First PTS   PTS length\n");
  totalsize=0;
  for (i=0;i<ngroups;i++) {
    for (j=0;j<ntracks[i];j++) {
      fprintf(stderr,"    %d     %02d  %10d  %10d  %10lld  ",i+1,j+1,files[i][j].first_sector,files[i][j].last_sector,files[i][j].first_PTS);
      fprintf(stderr,"%10lld\n",files[i][j].PTS_length);
    }
  }
  fprintf(stderr,"\n");

  return(0);
}
